package main

import (
	"context"
	"os"
	"os/signal"
	"time"

	corev1 "k8s.io/api/core/v1"
	"k8s.io/apimachinery/pkg/fields"
	icorev1 "k8s.io/client-go/informers/core/v1"
	"k8s.io/client-go/kubernetes"
	"k8s.io/client-go/rest"
	"k8s.io/client-go/tools/cache"
	"libre.sh/keydb-manager/internal"
)

func main() {
	ns := os.Getenv("NAMESPACE")
	instance := os.Getenv("INSTANCE")
	hostname := os.Getenv("HOSTNAME")

	config, err := rest.InClusterConfig()
	if err != nil {
		panic(err.Error())
	}
	clientset, err := kubernetes.NewForConfig(config)
	if err != nil {
		panic(err.Error())
	}

	informer := icorev1.NewPodInformer(clientset, ns, time.Second*30, cache.Indexers{cache.NamespaceIndex: cache.MetaNamespaceIndexFunc})
	manager := internal.NewManager()

	ctx, cancel := signal.NotifyContext(context.Background(), os.Interrupt)
	defer cancel()

	informer.AddEventHandler(cache.FilteringResourceEventHandler{
		FilterFunc: func(obj interface{}) bool {
			pod := obj.(*corev1.Pod)
			selector := fields.AndSelectors(
				fields.OneTermEqualSelector("app.kubernetes.io/name", "keydb"),
				fields.OneTermEqualSelector("app.kubernetes.io/instance", instance))
			f := fields.Set(pod.Labels)
			return selector.Matches(f) && pod.Name != hostname && pod.Status.PodIP != ""
		},
		Handler: cache.ResourceEventHandlerFuncs{
			AddFunc: func(obj interface{}) {
				pod := obj.(*corev1.Pod)
				manager.AddReplica(ctx, pod)
			},
			UpdateFunc: func(oldObj, newObj interface{}) {
				pod := newObj.(*corev1.Pod)
				replica := manager.GetReplica(pod.Name)
				if replica != pod.Status.PodIP {
					if replica != "" {
						manager.RemoveReplica(ctx, pod)
					}
					manager.AddReplica(ctx, pod)
				}
			},
			DeleteFunc: func(obj interface{}) {
				pod := obj.(*corev1.Pod)
				manager.RemoveReplica(ctx, pod)
			},
		}})

	informer.Run(make(<-chan struct{}))
}
